from .models.evo48 import Panel_EVO48
from .models.evo96 import Panel_EVO96
from .models.evo192 import Panel_EVO192
from .models.evohd import Panel_EVOHD

__all__ = ["Panel_EVO48", "Panel_EVO96", "Panel_EVO192", "Panel_EVOHD"]
